--[[
Created by: Adalan@Aruntas
Support, Fixes, Improvements by:


--]]

----------------------------------------
--- Contact me, before u make changes
--- or want to use code of this addon
---
--- at www.esoui.com                   
----------------------------------------

RecipeInfo = {}
-----------------------------------------
RecipeInfo.name = "RecipeInfo"
RecipeInfo.version = "1.03"
RecipeInfo.author = "Adalan@Aruntas"

local gPlayerChar = "unknown"
local ADDON_MANAGER = GetAddOnManager()
local gBlacklistetAddonFound = false

-- default values for saved variables
----------------------------------------- 
RecipeInfo.DefaultSettings = 
{
	-- include this char on recipe compares
	compareMyRecipes = true,
}

RecipeInfo.BlacklistetAddons = 
{
	"MasterThief",
}

RecipeInfo.KnownRecipes = {}

-- OnLoad-Handler
local function OnAddOnLoaded(event, addonName)
	if addonName == RecipeInfo.name then
	   RecipeInfo:Initialize()
	end
end 

-- Initialize
function RecipeInfo:Initialize()
	gBlacklistetAddonFound = RecipeInfo:IsAnyAddonBlacklistet()
	if (gBlacklistetAddonFound == true) then zo_callLater(function() d(GetString(IRK_MISC_ADDON_BLACKLISTED)) end, 1400) return end

	--set default or load SavedVariables
	RecipeInfo.SavedVarsSettings = ZO_SavedVars:New("RecipeInfoVars", 1, "Settings", RecipeInfo.DefaultSettings)
	RecipeInfo.AccountSavedVariables = ZO_SavedVars:NewAccountWide("RecipeInfoVars", 1, nil, RecipeInfo.KnownRecipes)

	
	EVENT_MANAGER:RegisterForEvent(RecipeInfo.name, EVENT_PLAYER_ACTIVATED, function()
		gPlayerChar = GetUnitName("player")
		RecipeInfo:SaveRemoveKnownRecipes()
		RecipeInfo:CreateSettingsMenu()
		RecipeInfo:InventarTooltip()
		RecipeInfo:LootTooltip()
		RecipeInfo:StoreTooltip()
		-- learn new recipes if any for SavedVars
		EVENT_MANAGER:RegisterForEvent(RecipeInfo.name, EVENT_RECIPE_LEARNED, RecipeInfo.RecipeLearned)
		EVENT_MANAGER:UnregisterForEvent(RecipeInfo.name, EVENT_PLAYER_ACTIVATED)
    end)
end	

------------------------------------------------------------------------------------
---- Direkt calls by EVENTS - START 
----------------------------------
-- a new recipe is learned
-- add it to the comparelist in SavedVars
function RecipeInfo.RecipeLearned(eventCode, recipeListIndex, recipeIndex)
	if (RecipeInfo.SavedVarsSettings.compareMyRecipes == true) then 
		RecipeInfo:SaveRemoveKnownRecipes()
	end
end 
----------------------------------
---- Direkt calls by EVENTS - END 
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
---- Subroutines - START 
----------------------------------
-- Save or remove all known recipes to/from comparepool for this char
--	
function RecipeInfo:SaveRemoveKnownRecipes()
	if (RecipeInfo.SavedVarsSettings.compareMyRecipes == true) then
		-- do update the recipe-list of savedvars with my known recipes
		local numRecipeLists = GetNumRecipeLists()
		local countAdded = 0

		for i = 1, numRecipeLists do
			local _Name, recipeListIndex = GetRecipeListInfo(i)
			for n = 1, recipeListIndex do
				local bKnown, recipeName, _, _, iQualityReq = GetRecipeInfo(i, n)
				local _lang = GetCVar("language.2")
				if (_lang == "de" or _lang == "fr") then 
					recipeName = RecipeInfo:FixLinkName(recipeName) 
				end
				
				if (bKnown) then -- known means here, that i read just all known recipes of my learned recipes
					-- proof recipe of own recipe list, whether its needed to add to SavedVars if not already in
					local nPos, bFound = RecipeInfo:IsKnownRecipeSaved(recipeName)
					if (bFound == false) then -- was not listed in savedvars
						RecipeInfo.AccountSavedVariables.KnownRecipes[nPos + 1] = {recipeName, {gPlayerChar} }
						countAdded = countAdded + 1
					else -- recipe is listet in savedvars
						-- add my charname, if not already added, because i got the same recipe
						local _arrPlayerchars = RecipeInfo:AddOrRemovePlayerchar(nPos, true) -- add playerchar to recipe list
						if (_arrPlayerchars ~= nil) then
							RecipeInfo.AccountSavedVariables.KnownRecipes[nPos] = {recipeName, _arrPlayerchars }
						end
					end
				end
			end
		end
		if (countAdded > 0) then d(countAdded..GetString(IRK_MISC_RECIPES_ADDED_TO_SEARCHPOOL)) end
	else --remove all known recipes, which do match by name
		local numRecipeLists = GetNumRecipeLists()
		local countRemoved = 0
		for i = 1, numRecipeLists do
			local _Name, recipeListIndex = GetRecipeListInfo(i)
			for n = 1, recipeListIndex do
				local bKnown, recipeName, _, _, iQualityReq = GetRecipeInfo(i, n)

				local _lang = GetCVar("language.2")
				if (_lang == "de" or _lang == "fr") then 
					recipeName = RecipeInfo:FixLinkName(recipeName) 
				end
				
				if (bKnown) then -- known means here, that i read just all known recipes of my learned recipes
					-- proof recipe of own recipe list, whether its needed to remove from list
					local nPos, bFound = RecipeInfo:IsKnownRecipeSaved(recipeName)
					if (bFound) then --remove it ?
						local _arrPlayerchars = RecipeInfo:AddOrRemovePlayerchar(nPos, false) -- remove playerCharname
						if (_arrPlayerchars == nil) then --remove recipe from savedvars list (if no charname is left)
							table.remove(RecipeInfo.AccountSavedVariables.KnownRecipes, nPos)
							countRemoved = countRemoved + 1
						else -- just update the list in savedvars
							RecipeInfo.AccountSavedVariables.KnownRecipes[nPos] = {recipeName, _arrPlayerchars }
						end
					end
				end
			end
		end
		if (countRemoved > 0) then d(countRemoved..GetString(IRK_MISC_RECIPES_REMOVED_FROM_SEARCHPOOL)) end
	end	
end

-- a new recipe is learned
-- add it to the comparelist in SavedVars
function RecipeInfo:CompareMyRecipes(eventCode, recipeListIndex, recipeIndex)
	if (RecipeSettings.compareMyRecipes) then 
		SaveRemoveKnownRecipes(nil, false)
	end
end

-- search recipename, whether it exists on compare-pool (searchpool)
function RecipeInfo:IsKnownRecipeSaved(_recipeName)
	if (RecipeInfo.AccountSavedVariables.KnownRecipes == nil) then
		RecipeInfo.AccountSavedVariables.KnownRecipes = {}
	end
	local nEntries = table.getn(RecipeInfo.AccountSavedVariables.KnownRecipes)	
	for idx, obj in ipairs(RecipeInfo.AccountSavedVariables.KnownRecipes) do
		local sRecipeName = obj[1]
		local arrPlayerCharnames = obj[2]
		if (string.lower(RecipeInfo:trim(sRecipeName)) == string.lower(RecipeInfo:trim(_recipeName))) then
			return idx, true, arrPlayerCharnames
		end	
	end
	return nEntries, false
end

-- search all recipes of the searchpool
function RecipeInfo:getNumRecipes()
	if (RecipeInfo.AccountSavedVariables.KnownRecipes == nil) then
		RecipeInfo.AccountSavedVariables.KnownRecipes = {}
	end
	local nEntries = table.getn(RecipeInfo.AccountSavedVariables.KnownRecipes)	
	return nEntries, false
end

-- Add or remove the playerchar from list
--
function RecipeInfo:AddOrRemovePlayerchar(nPos, bAdd)
	local obj = RecipeInfo.AccountSavedVariables.KnownRecipes[nPos]
	local _arrCharnames = obj[2]

	if (_arrCharnames == nil) then
		if (bAdd == false) then
			return nil -- no charname was found and table was empty (can get removed)
		elseif (bAdd) then -- initialize the array and add the actual playerCharname
			_arrCharnames = {}
			table.insert(_arrCharnames,gPlayerChar)
			return _arrCharnames
		end
	end

	if (bAdd) then -- add the actual Playercharname is wanted
		for idx, _Charname in ipairs(_arrCharnames) do
			if (_Charname == gPlayerChar and bAdd == true) then -- is already in list
				return nil 
			end
		end
		-- if no charname was found in list, which match with the actual charname then add me
		table.insert(_arrCharnames,gPlayerChar)
		return _arrCharnames		
	elseif (bAdd == false) then -- remove the actual Playercharname
		for idx, _Charname in ipairs(_arrCharnames) do
			if (_Charname == gPlayerChar and bAdd == false) then --remove playerchar
				table.remove(_arrCharnames,idx)
				if (table.getn(_arrCharnames) == 0) then return nil end
			end
		end
	end
	
	-- in case nothing was found, i give back the array
	return _arrCharnames
end

-- InventarTooltip hook routine
--
function RecipeInfo:InventarTooltip()
    local InventarItemTooltip = ItemTooltip.SetBagItem
    ItemTooltip.SetBagItem = function(control, iBagId, iSlotId)
		local itemLink = GetItemLink(iBagId, iSlotId, LINK_STYLE_DEFAULT)
		local itemType = GetItemLinkItemType(itemLink)
		InventarItemTooltip(control, iBagId, iSlotId)
		
		if (itemType == ITEMTYPE_RECIPE) then
			_recipeName = GetItemLinkName(GetItemLinkRecipeResultItemLink(itemLink))
			_recipeName = RecipeInfo:FixLinkName(_recipeName)
			local idx, bKnown, arrPlayerCharnames = RecipeInfo:IsKnownRecipeSaved(_recipeName)
			if (bKnown and arrPlayerCharnames ~= nil) then
				local _CharsKnown = ""
				for _, _Charname in ipairs(arrPlayerCharnames) do
					_CharsKnown = _CharsKnown.."|c00A5C6".._Charname.."|r, "
				end
				_CharsKnown = string.sub (_CharsKnown, 1, string.len(_CharsKnown)-2)
				ZO_Tooltip_AddDivider(control)
				control:AddLine("|cB552AD"..GetString(IRK_MISC_TOOLTIP_RECIPE_CHARS).."|r")
				control:AddLine(_CharsKnown, "ZoFontGame")
			end
		end
	end
end

-- StoreTooltip hook routine
function RecipeInfo:StoreTooltip()
    local StoreItemTooltip = ItemTooltip.SetStoreItem
    ItemTooltip.SetStoreItem = function(control, iInstancedLootId)
		local itemStoreLink = GetStoreItemLink(iInstancedLootId, LINK_STYLE_DEFAULT)
		local itemType = GetItemLinkItemType(itemStoreLink)
		StoreItemTooltip(control, iInstancedLootId)

		if (itemType == ITEMTYPE_RECIPE) then
			_recipeName = GetItemLinkName(GetItemLinkRecipeResultItemLink(itemStoreLink))
			_recipeName = RecipeInfo:FixLinkName(_recipeName)			
			local idx, bKnown, arrPlayerCharnames = RecipeInfo:IsKnownRecipeSaved(_recipeName)
			if (bKnown and arrPlayerCharnames ~= nil) then
				local _CharsKnown = ""
				for _, _Charname in ipairs(arrPlayerCharnames) do
					_CharsKnown = _CharsKnown.."|c00A5C6".._Charname.."|r, "
				end
				_CharsKnown = string.sub (_CharsKnown, 1, string.len(_CharsKnown)-2)
				ZO_Tooltip_AddDivider(control)
				control:AddLine("|cB552AD"..GetString(IRK_MISC_TOOLTIP_RECIPE_CHARS).."|r")
				control:AddLine(_CharsKnown, "ZoFontGame")
			end
		end
	end
end

-- LootTooltip hook routine
function RecipeInfo:LootTooltip()
    local LootItemTooltip = ItemTooltip.SetLootItem
    ItemTooltip.SetLootItem = function(control, iInstancedLootId)
		local itemLootLink = GetLootItemLink(iInstancedLootId, LINK_STYLE_DEFAULT)
		local itemType = GetItemLinkItemType(itemLootLink)
		LootItemTooltip(control, iInstancedLootId)

		if (itemType == ITEMTYPE_RECIPE) then
			_recipeName = GetItemLinkName(GetItemLinkRecipeResultItemLink(itemLootLink))
			_recipeName = RecipeInfo:FixLinkName(_recipeName)			
			local idx, bKnown, arrPlayerCharnames = RecipeInfo:IsKnownRecipeSaved(_recipeName)
			if (bKnown and arrPlayerCharnames ~= nil) then
				local _CharsKnown = ""
				for _, _Charname in ipairs(arrPlayerCharnames) do
					_CharsKnown = _CharsKnown.."|c00A5C6".._Charname.."|r, "
				end
				_CharsKnown = string.sub (_CharsKnown, 1, string.len(_CharsKnown)-2)
				ZO_Tooltip_AddDivider(control)
				control:AddLine("|cB552AD"..GetString(IRK_MISC_TOOLTIP_RECIPE_CHARS).."|r")
				control:AddLine(_CharsKnown, "ZoFontGame")
			end
		end
	end
end

-- Checkout for a blacklistet addon
--
function RecipeInfo:IsAnyAddonBlacklistet()
	for i = 1, ADDON_MANAGER:GetNumAddOns() do
		local addonName, _, _, _, bEnabled = ADDON_MANAGER:GetAddOnInfo(i)
		for n=1, table.getn(RecipeInfo.BlacklistetAddons) do
			if (addonName == RecipeInfo.BlacklistetAddons[n] and bEnabled == true) then return true end	
		end
	end	
	return false	
end
----------------------------------
---- Subroutines - END 
------------------------------------------------------------------------------------ 
------------------------------------------------------------------------------------
---- HELPFUL ROUTINES - START 
----------------------------------
function RecipeInfo:trim(s)
  return (s:gsub("^%s*(.-)%s*$", "%1"))
end

-- Fix the i.e ^n:m useless info on names or items
--
function RecipeInfo:FixLinkName(sString)
	return zo_strformat(SI_TOOLTIP_ITEM_NAME, sString)
end


----------------------------------
---- HELPFUL ROUTINES - END 
------------------------------------------------------------------------------------ 


-- Register Events for ADDON LOAD --
EVENT_MANAGER:RegisterForEvent(RecipeInfo.name, EVENT_ADD_ON_LOADED, OnAddOnLoaded)